@@ -1,6 +1,6 @@
 # local.py
 #
-# Copyright 2020-2024 Fabio Comuni, et al.
+# Copyright 2020-2025 Fabio Comuni, et al.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -171,16 +171,17 @@ def opendb(url, is_online=True):
 
     dbfile = dbfilename
 
-    from .models import Meta
-
-    is_time_to_update = (time.time() - Meta().last_update) > Settings.instance().get_event_cache()
-
-    if is_online and (migrated or is_time_to_update):
+    if check_need_update(is_online, migrated):
         return updatedb(url)
 
     return None
 
 
+def check_need_update(is_online=True, migrated=False):
+    is_time_to_update = (time.time() - models.Meta().last_update) > Settings.instance().get_event_cache()
+    return is_online and (migrated or is_time_to_update)
+
+
 def updatedb(from_url):
     """Update event cache from schedule url
 
@@ -407,3 +408,4 @@ def clear_cache():
                 fp = os.path.join(dirpath, f)
                 if fp != dbfile:
                     os.remove(fp)
+
