@@ -1,6 +1,6 @@
 # mainwindow.py
 #
-# Copyright 2020-2024 Fabio Comuni, et al.
+# Copyright 2020-2025 Fabio Comuni, et al.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ from typing import Optional
 import json
 from time import time
 from gettext import gettext as _
+from enum import IntEnum
 
 from gi.repository import GLib
 from gi.repository import Gio
@@ -35,20 +36,11 @@ from .config import APP_NAME, APP_ID  # type:ignore # config is generated at com
 from .networkconnectionmonitor import NetworkConnectionMonitor
 from .widgets import LoaderPage
 from .widgets import Navbar
-from .widgets.navbar import NavbarItem
-from .pages import PageInfo
-from .pages import PageList
-from .pages import PageEvents
+from .widgets import LoadingOverlay
+from .widgets.navbar import NavbarListItem
+from .pages import PageInfo, PageList, PageEvents, PageDetails
 from .utils import clean_markup
 
-PAGES = dict(
-    info=NavbarItem(name="info", label=_("Event info"), icon="emblem-system-symbolic"),
-    days=NavbarItem(name="days", label=_("Days"), icon="x-office-calendar-symbolic"),
-    tracks=NavbarItem(name="tracks", label=_("Tracks"), icon="format-justify-fill-symbolic"),
-    rooms=NavbarItem(name="rooms", label=_("Rooms"), icon="display-projector-symbolic"),
-    starred=NavbarItem(name="starred", label=_("Starred"), icon="starred-symbolic", group=1)
-)
-
 
 @Gtk.Template(resource_path="/net/kirgroup/confy/mainwindow.ui")
 class MainWindow(Adw.ApplicationWindow):
@@ -91,23 +83,30 @@ class MainWindow(Adw.ApplicationWindow):
     # Keeps Notification ojects around to be able to get activation events
     notifications: dict[str, Notify.Notification] = {}
 
+    # we use the app timer "tick" event to automatically update cached data
+    # but: we don't want to un update while an update is already running
+    an_update_is_running = False
+
     def __init__(self, application: Gtk.Application, **kwargs):
         super().__init__(application=application, **kwargs)
 
         size = Settings.instance().get_size()
         self.set_default_size(*size)
+        self.props.maximized = Settings.instance().get_maximized()
 
         # Check if there are notifications to send
         application.connect('tick', self.check_event_notification)
 
+        application.connect('tick', self.auto_update_conf)
+
         # handle conf data updating, eg. reloading from remote
         self.connect('conf-updated', self._on_conf_updated)
 
-        self._add_action("open-conference", None, ["<Primary>o",], self.open_conference)
-        self._add_action("search", None, ["<Primary>f",], lambda *_: self.show_search_all())
-        self._add_action("copy", None, ["<Primary>c",], lambda *_: self.copy_details_to_clipboard())
+        self._add_action("open-conference", None, None, ["<Primary>o",], self.open_conference)
+        self._add_action("search", None, GLib.Variant.new_boolean(False), ["<Primary>f",], None, self._on_search_action).set_enabled(False)
+        self._add_action("copy", None, None, ["<Primary>c",], lambda *_: self.copy_details_to_clipboard()).set_enabled(False)
         # open recent action, used by recent menu
-        self._add_action("open-recent", "i", None, self._on_recent_action)
+        self._add_action("open-recent", "i", None, None, self._on_recent_action)
 
         # TODO: serve?
         self.nm.connect("notify", self._on_networkconnectionmonitor_notify)
@@ -119,12 +118,20 @@ class MainWindow(Adw.ApplicationWindow):
         self.recentmanager.connect("changed", self._on_recent_changed)
         self._on_recent_changed()
 
-    def _add_action(self, name, parameter_type, accels, activate_cbk):
+    def _add_action(self, name, parameter_type, state, accels, activate_cbk, state_cbk = None):
         """Simplify adding an action to the window"""
         if parameter_type is not None:
             parameter_type = GLib.VariantType.new(parameter_type)
-        action = Gio.SimpleAction.new(name, parameter_type)
-        action.connect("activate", activate_cbk)
+        action = Gio.SimpleAction(
+            name=name,
+            parameter_type=parameter_type,
+            state=state
+        )
+
+        if activate_cbk:
+            action.connect("activate", activate_cbk)
+        if state_cbk:
+            action.connect("notify::state", state_cbk)
         self.add_action(action)
         if accels is not None:
             self.get_application().set_accels_for_action(F"win.{name}", accels)
@@ -212,12 +219,11 @@ class MainWindow(Adw.ApplicationWindow):
         """
         Callback when conference data is modified (eg. is reloaded)
         """
-
-        self.navbar.set_item_cond(PAGES['info'], True)
-        self.navbar.set_item_cond(PAGES['days'], models.Day.count() > 0)
-        self.navbar.set_item_cond(PAGES['tracks'], models.Track.count() > 0)
-        self.navbar.set_item_cond(PAGES['rooms'], models.Room.count() > 0)
-        self.navbar.set_item_cond(PAGES['starred'], True)
+        self.an_update_is_running = False
+        self.lookup_action("search").set_enabled(True)
+        self.navbar.days.props.visible = models.Day.count() > 0
+        self.navbar.tracks.props.visible = models.Track.count() > 0
+        self.navbar.rooms.props.visible = models.Room.count() > 0
 
     def _on_fetcher_done(self, *_):
         """
@@ -240,21 +246,40 @@ class MainWindow(Adw.ApplicationWindow):
         print(notification, action, data)
         if action == "view" and data.startswith("nextup-"):
             self.present_with_time(time())
-            eventid = int(data.split("-")[1])
+            eventid = data.split("-", maxsplit=1)[1]
             event = models.Event.by_id(eventid)
-            # if we are not on "events" subpage, force it as non navigatable,
-            # we want to be back to where we were
-            print(self.subpage_leaflet.get_visible_child(), self.subpage_events)
-            can_we_go_back_to_events = self.subpage_leaflet.get_visible_child() == self.subpage_events
-            self.show_details(event, can_we_go_back_to_events)
+            page = PageDetails(obj=event, tagfmt="notif-{0.id}")
+            self.main_stack.replace([page])
 
     @Gtk.Template.Callback()
-    def _on_page_changed(self, navbar):
-        self.navigate_to(navbar.get_current_item().name)
+    def _on_page_changed(self, navbar: Navbar, navitem: NavbarListItem):
+        self.navigate_to(navitem)
 
     @Gtk.Template.Callback()
     def _on_close_requested(self, *args):
-        print("close-request", args)
+        size = self.get_default_size()
+        Settings.instance().set_size(size)
+        Settings.instance().set_maximized(self.props.maximized)
+
+    @Gtk.Template.Callback()
+    def _on_stack_replaced(self, stack):
+        self.lookup_action("copy").set_enabled(False)
+        page = stack.get_visible_page()
+        if hasattr(page, "ready"):
+            page.ready()
+
+    @Gtk.Template.Callback()
+    def _on_stack_page_pushed(self, stack):
+        if isinstance(stack.get_visible_page(), PageDetails):
+            self.lookup_action("copy").set_enabled(True)
+
+    @Gtk.Template.Callback()
+    def _on_stack_page_popped(self, stack, page):
+        self.lookup_action("copy").set_enabled(False)
+        print("pop", page.props.tag)
+        if page.props.tag == "search":
+            print("set search state False")
+            self.lookup_action("search").set_state(GLib.Variant.new_boolean(False))
 
     def open_conference(self, *_):
         """
@@ -277,9 +302,10 @@ class MainWindow(Adw.ApplicationWindow):
         # So existing pages will update values but with old conf object.
         # Reset navigation will force to create a new PageInfo with the new
         # conf object
-        self.navigate_to(None)  # reset navigation
+        self.navigate_to_none()
 
         self.show_loader()
+        self.an_update_is_running = True
         try:
             fetcher = local.openconf(conf, is_online=self.nm.get_isconnected())
         except Exception as e:
@@ -304,6 +330,30 @@ class MainWindow(Adw.ApplicationWindow):
         self.notifications = {}
         self.check_event_notification()
 
+    def auto_update_conf(self, *args):
+        """
+        Update cache automatically when it expires (if we are online)
+        """
+        if self.conf is None:
+            # no conf loaded, nothing to do
+            return
+
+        if self.an_update_is_running:
+            # an update is running, nothing to do
+            return
+
+        def _on_f_done(*args):
+            self.emit("conf_updated")
+            toast.dismiss()
+
+        is_online = self.nm.get_isconnected();
+        if is_online and local.check_need_update():
+            self.an_update_is_running = True
+            f = local.updatedb(self.conf.url)
+            f.connect("done", _on_f_done)
+            toast = LoadingOverlay(f, _("Updating schedule..."))
+            self.overlay.add_toast(toast)
+
     def check_event_notification(self, *args):
         """
         Notifiy starred talk staring in next 5 minutes
@@ -311,15 +361,20 @@ class MainWindow(Adw.ApplicationWindow):
         if self.conf is None:
             # no conf loaded, nothing to do
             return
+
         nextup = list(models.Event.nextup(5))
         for e in nextup:
+            description = _("at {}").format(e.start_in_tz().strftime("%H:%M"))
+            if e.room:
+                description = _("at {} in {}").format(
+                    e.start_in_tz().strftime("%H:%M"),
+                    e.room
+                )
+
             title = clean_markup(e.title)
             self._send_desktop_notification(
                 _("Next up: {}").format(title),
-                _("at {} in {}").format(
-                    e.start_in_tz().strftime("%H:%M"),
-                    e.room
-                ),
+                description,
                 f"nextup-{e.id}")
             e.set_notified(True)
 
@@ -344,14 +399,16 @@ class MainWindow(Adw.ApplicationWindow):
 
     def show_main(self):
         """Show 'main' page"""
-        self.navigate_to("info")
+        self.navigate_to(self.navbar.info)
         self.loader_stack.set_visible_child_name("main")
 
-    def navigate_to(self, name: Optional[str]):
+    def navigate_to_none(self):
+        """Remove everything from main stack"""
+        self.main_stack.replace([Adw.NavigationPage()])
+
+    def navigate_to(self, info: NavbarListItem):
+        name = info.get_buildable_id()
         print("Navigate to", name)
-        if name is None:
-            self.main_stack.replace([Adw.NavigationPage()])
-            return
 
         # we already into this navigation path?
         top = self.main_stack.find_page(name)
@@ -360,26 +417,27 @@ class MainWindow(Adw.ApplicationWindow):
             if self.main_split_view.get_collapsed():
                 self.main_split_view.set_show_sidebar(False)
             return
-
+        else:
         # no, start anew
-        child: Adw.NavigationPage
-        match name:
-            case "info":
-                info = PAGES[name]
-                child = PageInfo(self, self.conf, title=info.label, tag=name)
-            case "starred" | "search":
-                child = PageEvents(None, model=name, tag=name)
-            case _:
-                info = PAGES[name]
-                child = PageList(self, self.conf, model=name, title=info.label, tag=name)
+            child: Adw.NavigationPage
+            match name:
+                case "info":
+                    child = PageInfo(self, self.conf, title=info.label, tag=name)
+                case "starred":
+                    child = PageEvents(None, model=name, tag=name)
+                case _:
+                    child = PageList(self, self.conf, model=name, title=info.label, tag=name)
+            self.main_stack.replace([child])
+
+        # set navbar selection
+        self.navbar.select_row(info)
 
-        self.main_stack.replace([child])
         if self.main_split_view.get_collapsed():
             self.main_split_view.set_show_sidebar(False)
 
     def copy_details_to_clipboard(self):
         page = self.main_stack.get_visible_page()
-        if page and page.get_tag() == "details":
+        if isinstance(page, PageDetails):
             page.copy_to_clipboard()
             self.notification_show(_("Event details copied to the clipboard"))
 
@@ -388,5 +446,25 @@ class MainWindow(Adw.ApplicationWindow):
         t.set_title(text)
         self.overlay.add_toast(t)
 
-    def show_search_all(self):
-        self.navigate_to("search")
+    def _on_search_action(self, action, param):
+        print("search state is", action.get_state())
+        page = self.main_stack.find_page("search")
+        if action.get_state():
+            # if state is true, push "search" page to stack
+            if not page:
+                page = PageEvents(None, model="search", tag="search")
+            self.main_stack.push(page)
+        else:
+            # if state is false, and search page is in stack,
+            # pop to the page before in the stack
+            if page:
+                prevpage = self.main_stack.get_previous_page(page)
+                # Note: 'prevpage' should never be None, because search page should
+                # never be first in stack, and because if we are here _is_ in the stack
+                # but better safe than sorry
+                if prevpage:
+                    self.main_stack.pop_to_page(prevpage)
+
+        if self.main_split_view.get_collapsed():
+            self.main_split_view.set_show_sidebar(False)
+
