@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-from typing import Optional, Any
+from typing import Optional, Any, Union
 import datetime
 
 from gi.repository import GObject, Gtk
@@ -28,14 +28,20 @@ class ObjectNotFound(Exception):
     pass
 
 
+DayOrDateOrStr = Optional[Union["Day", str, datetime.date]]
+RoomOrName = Optional[Union["Room", str]]
+TrackOrName = Optional[Union["Track", str]]
+EventOrId = Optional[Union["Track", str]]
+
+
 # models
 def _build_filter(
         table: str = "",
         *,
-        day: Optional["Day"] = None,
-        room: Optional["Room"] = None,
-        track: Optional["Track"] = None,
-        event: Optional["Event"] = None,
+        day: DayOrDateOrStr = None,
+        room: RoomOrName = None,
+        track: TrackOrName = None,
+        event: EventOrId = None,
         **kwargs):
     filterdata = []
     wherestm = []
@@ -50,14 +56,31 @@ def _build_filter(
             wherestm.append(f"{table}{col} = ?")
             filterdata.append(value)
 
-    if day is not None:
-        __f("date", day.date)
-    if room is not None:
-        __f("room", room.name)
-    if track is not None:
-        __f("track", track.name)
-    if event is not None:
-        __f("id", event.id)
+    match day:
+        case str():
+            __f("date", day)
+        case datetime.date():
+            __f("date", day)
+        case Day():
+            __f("date", day.date)
+
+    match room:
+        case str():
+            __f("room", room)
+        case Room():
+            __f("room", room.name)
+
+    match track:
+        case str():
+            __f("track", track)
+        case Track():
+            __f("track", track.name)
+
+    match event:
+        case str():
+            __f("id", event)
+        case Event():
+            __f("id", event.id)
 
     for k, v in kwargs.items():
         __f(k, v)
@@ -453,7 +476,7 @@ class Event(GObject.GObject):
 
     @classmethod
     def filter(cls,
-               day: Optional[Day] = None,
+               day: Optional[Union[Day, str]] = None,
                room: Optional[Room] = None,
                track: Optional[Track] = None,
                order_by: str = "start, room",
@@ -479,8 +502,8 @@ class Event(GObject.GObject):
         # now = "now" # is "now" somehow broken?
         query = """SELECT * FROM events
                     WHERE starred=1 AND notified=0
-                        AND datetime(start) >= datetime("{1}")
-                        AND datetime(start) <= datetime("{1}", "+{0} minutes");"""
+                        AND datetime(start) >= datetime('{1}')
+                        AND datetime(start) <= datetime('{1}', '+{0} minutes');"""
         for row in local.getDb().execute(query.format(minutes, now)):
             yield cls()._init(row)
 
