@@ -102,7 +102,10 @@ class OpenWindow(Adw.ApplicationWindow):
     def delete_conf(self, conf, cbk=None):
         def _message_response(dialog, response):
             if response == "delete":
+                # delete from user menu
                 local.delete_user_menu(conf.to_json())
+                # delete from recent list
+                Gtk.RecentManager.get_default().remove_item(conf.url)
                 self.update_list()
                 if cbk is not None:
                     cbk(conf)
