@@ -1,6 +1,6 @@
 # events.py
 #
-# Copyright 2020-2024 Fabio Comuni, et al.
+# Copyright 2020-2025 Fabio Comuni, et al.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,56 @@ from ..widgets import EventActionRow, Author, StarButton
 from ..widgets import WebLinkActionRow, weblinkactionrowactivated
 from ..models import Event
 
+# Detail page can link to other detail page via the "overlapping events" list
+# Overlapping events are circular (as in event B overlaps A , so event A overlaps B)
+# so just pushing pages to the stack will can get the user to a very long stack:
+#   List -> Event A -> Event B -> A -> B -> A -> B
+#
+# So we use tags to find if a detail page is already in stack and pop back to that:
+#   List -> Event A -> Event B
+#       user clicks on overlapping Event A, goes back to
+#   List -> Event A
+#
+# The tag name is based on 'tagfmt' propery, which defaults to 'TAG_FMT'.
+#
+# But then there is search.
+# If an user is on a detail page and trigger search, the search page is pushed
+# to the stack and from there any result will push a detail page.
+#   List -> Event A -> Search -> Event C
+#
+# And the user can look at results and go back to were it was gout out from search
+# (either navigating back the stack or triggering the 'search' action)
+#
+# But if user in search page clicks on an event which detail page is already on
+# the stack, it will popped back before the search page
+#   List -> Event A -> Search
+#      user clicks on Avent A, then goes to
+#   List -> Event A
+#
+# Which exit the 'search' state (the togglebutton in side headbar deactivate) and is
+# inconsistent with previous search navigation behaviour
+#
+# So detail pages created from search gets different 'tagfmt' value (see 'PageEvents::on_litstbox_activated')
+# this will keep the navigation behaviour of going back to already pushed detail page clicking
+# on overlapping events, but keeping the user in the 'search state'
+#   List -> Event A -> Search
+#      user clicks on Avent A, then goes to
+#   List -> Event A -> Search -> Event A
+#       clicks on overlapping Event B
+#   List -> Event A -> Search -> Event A -> Event B
+#       clicks on overlapping Event A
+#   List -> Event A -> Search -> Event A
+#       goes back
+#   List -> Event A -> Search
+#       goes back
+#   List -> Event A
+#
+# Also, when user clicks on incoming event notification, the detail page for the
+# event is pushed with yet another 'tagfmt' (see 'MainWindow::_on_notification_action')
+#
+
+TAG_FMT = "details-{0.id}"
+
 
 @Gtk.Template(resource_path="/net/kirgroup/confy/pages/details.ui")
 class PageDetails(Adw.NavigationPage):
@@ -37,6 +87,8 @@ class PageDetails(Adw.NavigationPage):
     links_list: Gtk.ListBox = Gtk.Template.Child()  # type: ignore
     author_box: Gtk.FlowBox = Gtk.Template.Child()  # type: ignore
 
+    tagfmt: str = GObject.property(type=str, default=TAG_FMT)
+
     _subtitle = ""
     _authors = ""
     _time = ""
@@ -138,6 +190,8 @@ class PageDetails(Adw.NavigationPage):
         self._on_obj_update()
 
     def _on_obj_update(self, *_):
+        self.props.tag = self.tagfmt.format(self.obj)
+        print(f"PageDetails obj_update {self.obj} tag:{self.props.tag}")
         self.props.title = markup_to_text(self.obj.title)
         self.props.authors = ", ".join([clean_markup(str(p)) for p in self.obj.persons()])
         self.props.subtitle = markup_to_text(self.obj.subtitle)
@@ -210,7 +264,16 @@ class PageDetails(Adw.NavigationPage):
 
     @Gtk.Template.Callback()
     def on_conflict_activated(self, listbox, row):
-        self.set_obj(row.obj)
+        obj = row.obj
+        tag = self.tagfmt.format(obj)
+        stack = self.get_parent()
+        page = stack.find_page(tag)
+        print(obj, tag, page)
+        if page:
+            stack.pop_to_page(page)
+        else:
+            page = PageDetails(obj=obj, tagfmt=self.tagfmt)
+            stack.push(page)
 
     @Gtk.Template.Callback()
     def on_link_activated(self, listbox, row):
