@@ -1,6 +1,6 @@
 # events.py
 #
-# Copyright 2020-2024 Fabio Comuni, et al.
+# Copyright 2020-2025 Fabio Comuni, et al.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ from gi.repository import Gio
 from gi.repository import Gtk
 from gi.repository import Adw
 
-from .details import PageDetails
+from .details import PageDetails, TAG_FMT
 from .. import models
 from ..config import FMT_DAY  # type:ignore # config is generated at compile time
 from ..utils import clean_markup
@@ -60,7 +60,6 @@ class PageEvents(Adw.NavigationPage):
     _order_by: str = "start, room"
 
     _update_handler_id: Optional[int] = None
-    _value: Optional[models.Event] = None
 
     timeout = InterruptibleTimeout()
 
@@ -126,6 +125,9 @@ class PageEvents(Adw.NavigationPage):
         group.insert(action)
         self.insert_action_group("list", group)
 
+    def ready(self):
+        pass
+
     def _on_group_by(self, action: Gio.SimpleAction, param: GLib.Variant):
         self.set_group_by(param.get_string())
         action.set_state(param)
@@ -133,11 +135,8 @@ class PageEvents(Adw.NavigationPage):
         with models.Meta() as m:
             m.set(f"{self._model}_group_by", param.get_string())
 
-    def get_value(self):
-        return self._value
-
     def set_value(self, obj):
-        print(f"PageEvents: set_value(obj={obj}) model={self._model}")
+        # print(f"PageEvents: set_value(obj={obj}) model={self._model}")
         if self._update_handler_id:
             models.dbUpdateWatcher.disconnect(self._update_handler_id)
         self._update_handler_id = None
@@ -192,6 +191,7 @@ class PageEvents(Adw.NavigationPage):
             # self.search_button.props.visible = False
             self.searchbar.props.search_mode_enabled = True
             self.searchbar.props.show_close_button = False
+            self.connect("shown", lambda *_: self.searchbar.grab_focus())
             self.filters = None
             self.set_group_by(None)
 
@@ -323,6 +323,6 @@ class PageEvents(Adw.NavigationPage):
             self.stack.set_visible_child_name('list')
 
     def on_litstbox_activated(self, listbox, row):
-        self._value = value = row.obj
-        page = PageDetails(obj=value)
+        tagfmt = "search-{0.id}" if self._model == "search" else TAG_FMT
+        page = PageDetails(obj=row.obj, tagfmt=tagfmt)
         self.get_parent().push(page)
