@@ -1,6 +1,6 @@
 # ics.py
 #
-# Copyright 2020-2022 Fabio Comuni, et al.
+# Copyright 2020-2025 Fabio Comuni, et al.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@ from icalendar import Calendar  # type: ignore
 
 from .. import local
 from ..models import Meta
+from ..utils import bind_list
 
 
 def import_ics(content: str, url: str):
@@ -40,6 +41,8 @@ def import_ics(content: str, url: str):
     meta_end = datetime(1970, 1, 1, 0, 0, 0)
     meta_start = datetime(3100, 12, 12, 23, 59, 59)
 
+    found_event_ids = []
+
     for c in cal.walk():
         if c.name == "VCALENDAR":
             meta_title = c.get('X-WR-CALNAME')
@@ -47,6 +50,7 @@ def import_ics(content: str, url: str):
         if c.name == "VEVENT":
             fulltextsearch = []
             eventid = c['UID']
+            found_event_ids.append(eventid)
             if c.get('DTSTART'):
                 start = c['DTSTART'].dt.astimezone(UTC).replace(tzinfo=None)
             else:
@@ -103,6 +107,9 @@ def import_ics(content: str, url: str):
                                 (event_id, href, name) VALUES (?,?,?)""", links)
             fulltextsearchstr = ' '.join(fulltextsearch)
             _db.execute("""INSERT OR REPLACE INTO fts_event (event_id, text) VALUES (?, ?)""", (eventid, fulltextsearchstr))
+
+    _db.execute(f"""DELETE FROM events WHERE id NOT IN ({bind_list(found_event_ids)})""", found_event_ids)
+    _db.execute(f"""DELETE FROM fts_event WHERE event_id NOT IN ({bind_list(found_event_ids)})""", found_event_ids)
     _db.commit()
 
     with Meta() as m:
