@@ -1,6 +1,6 @@
 # pentabarf.py
 #
-# Copyright 2020-2022 Fabio Comuni, et al.
+# Copyright 2020-2025 Fabio Comuni, et al.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ from dateutil.tz import UTC, tzlocal
 from .exceptions import InvalidFormatException
 from .. import local
 from ..models import Meta
+from ..utils import bind_list
 
 
 def _local_to_utc_to_unset(dt):
@@ -69,6 +70,9 @@ def import_pentabarf(xmlstr: str, url: str):
 
     _db = local.getDb()
 
+    found_event_ids = []
+    found_person_ids = []
+
     for eday in root.iter('day'):
         date = eday.attrib['date']
         # found in sfscon. events nested in <room name=""> element
@@ -78,12 +82,17 @@ def import_pentabarf(xmlstr: str, url: str):
         if rooms:
             for eroom in rooms:
                 room = eroom.attrib['name']
-                _handle_events(_db, eroom, base_url, date, room)
+                feids, fpids = _handle_events(_db, eroom, base_url, date, room)
+                found_event_ids += feids
+                found_person_ids += fpids
         else:
             # if we don't find 'day > room', we then parse as 'day > event'
-            _handle_events(_db, eday, base_url, date)
+            found_event_ids, found_person_ids = _handle_events(_db, eday, base_url, date)
 
-    # TODO: remove removed events
+    # cleanup removed events and persons
+    _db.execute(f"""DELETE FROM persons WHERE id NOT IN ({bind_list(found_person_ids)})""", found_person_ids)
+    _db.execute(f"""DELETE FROM events WHERE id NOT IN ({bind_list(found_event_ids)})""", found_event_ids)
+    _db.execute(f"""DELETE FROM fts_event WHERE event_id NOT IN ({bind_list(found_event_ids)})""", found_event_ids)
 
     _db.commit()
 
@@ -93,7 +102,10 @@ def _handle_events(
         root: ET.Element,
         base_url: str,
         date: str,
-        room: Optional[str] = None) -> None:
+        room: Optional[str] = None) -> tuple[list, list]:
+
+    found_event_ids = []
+    found_person_ids = []
 
     for eevent in root.iter('event'):
         fulltextsearch = []
@@ -104,6 +116,7 @@ def _handle_events(
             eventid = eevent.attrib['unique_id']
         else:
             eventid = eevent.attrib['id']
+        found_event_ids.append(eventid)
 
         tstart = _get_text(eevent, 'start').strip()[:5]
         start = _local_to_utc_to_unset(
@@ -138,6 +151,7 @@ def _handle_events(
             persons.append((
                 id, name, org, thumbnail, bio, url
             ))
+            found_person_ids.append(id)
 
         fulltextsearch += [s for s in [title, subtitle, abstract, description, room, track] if s is not None]
         fulltextsearch += [p[1] for p in persons]
@@ -180,3 +194,5 @@ def _handle_events(
 
         fulltextsearchstr = ' '.join(fulltextsearch)
         _db.execute("""INSERT OR REPLACE INTO fts_event (event_id, text) VALUES (?, ?)""", (eventid, fulltextsearchstr))
+
+    return found_event_ids, found_person_ids
