@@ -4,6 +4,11 @@ from html.parser import HTMLParser
 from gi.repository import GLib
 
 
+def bind_list(lst: list) -> str:
+    "Small util to bind list in a query"
+    return ",".join("?" * len(lst))
+
+
 def convert_markdown_links(text):
     """converts markdown links to html hyperlinks"""
     return re.sub(r"\[([^]]+?)\]\(([^)\s]+?)(?:\s.*?)?\)", r'<a href="\2">\1</a>', text)
@@ -23,7 +28,7 @@ def clean_markup(text):
 
 def markup_to_text(text):
     """return plain text version of markup string"""
-    text = clean_markup(text)
+    text = clean_markup(f"<div>{text}</div>")
     text = re.sub(r'</?i>', '_', text)
     text = re.sub(r'</?tt>', '`', text)
     text = re.sub(r'</?b>', '**', text)
